use std::{collections::HashMap, fs, io::ErrorKind, path::PathBuf};

use log::{debug, error};
use serde::Deserialize;

mod enumeration;

#[derive(Deserialize)]
pub struct Show {
    pub path: PathBuf,
    pub tags: Vec<String>,
}

#[derive(Debug, Eq, Hash, PartialEq)]
pub enum EpisodeNumber {
    Standalone,
    SingleSeason(u32),
    MultiSeason(u32, u32),
}

type Episodes = HashMap<EpisodeNumber, PathBuf>;

pub fn load(shows_file: PathBuf) -> HashMap<String, Show> {
    serde_yaml::from_reader(fs::File::open(shows_file).expect("Failed to open shows file"))
        .expect("Failed to parse YAML from shows file")
}

impl Show {
    pub fn episodes(&self) -> std::io::Result<Episodes> {
        let path = &self.path;
        let metadata = fs::metadata(path)?;
        if metadata.is_file() {
            debug!("{} is a file, standalone", path.display());
            Ok(HashMap::from([(
                EpisodeNumber::Standalone,
                path.to_path_buf(),
            )]))
        } else if metadata.is_dir() {
            debug!("{} is a directory, enumerating episodes", path.display());
            let files: Vec<PathBuf> = fs::read_dir(path)?
                .map(|entry| {
                    let entry = entry?;
                    if !entry.file_type()?.is_file() {
                        debug!("Skipping {}, not a file", entry.path().display());
                        return Ok(None);
                    }
                    if entry.file_name().into_string().is_err() {
                        error!(
                            "Path {} contains invalid unicode, skipping",
                            entry.path().display()
                        );
                        return Ok(None);
                    }
                    Ok(Some(entry.path()))
                })
                .filter_map(|r| r.transpose())
                .collect::<std::io::Result<Vec<PathBuf>>>()?;
            enumeration::enumerate_episodes(files).ok_or(std::io::Error::new(
                ErrorKind::InvalidData,
                "No valid prefixes found",
            ))
        } else {
            Err(std::io::Error::new(
                ErrorKind::InvalidInput,
                format!("Invalid file type for {}", path.display()),
            ))
        }
    }
}
